@@ -35,6 +35,16 @@ appDependencies <- function(project = NULL,
 
   project <- getProjectDir(project)
 
+  ## We want to search both local and global library paths for DESCRIPTION files
+  ## in the recursive dependency lookup; hence we take a large (ordered) union
+  ## of library paths. The ordering ensures that we search the private library first,
+  ## and fall back to the local / global library (necessary for `packrat::init`)
+  libPaths <- c(
+    libDir(project),
+    .libPaths(),
+    .packrat_mutables$origLibPaths
+  )
+
   ## For R packages, we only use the DESCRIPTION file
   if (isRPackage(project)) {
 
@@ -42,24 +52,35 @@ appDependencies <- function(project = NULL,
     parentDeps <-
       pkgDescriptionDependencies(file.path(project, "DESCRIPTION"))$Package
 
+    # Strip out any dependencies the user has requested we do not track.
+    parentDeps <- setdiff(parentDeps, packrat::opts$ignored.packages())
+
     ## For downstream dependencies, we don't grab their Suggests:
     ## Presumedly, we can build child dependencies without vignettes, and hence
     ## do not need suggests -- for the package itself, we should make sure
     ## we grab suggests, however
     childDeps <- recursivePackageDependencies(parentDeps,
-                                              libDir(project),
+                                              libPaths,
                                               available.packages,
                                               fields)
   } else {
     parentDeps <- setdiff(unique(c(dirDependencies(project))), "packrat")
+    parentDeps <- setdiff(parentDeps, packrat::opts$ignored.packages())
     childDeps <- recursivePackageDependencies(parentDeps,
-                                              libDir(project),
+                                              libPaths,
                                               available.packages,
                                               fields)
   }
 
-  sort_c(unique(c(parentDeps, childDeps, "packrat")))
+  result <- unique(c(parentDeps, childDeps, "packrat"))
 
+  # If this project is implicitly a shiny application, then
+  # add that in as the previously run expression dependency lookup
+  # won't have found it.
+  if (!("shiny" %in% result) && isShinyApp(project))
+    result <- c(result, "shiny")
+
+  sort_c(result)
 }
 
 # detect all package dependencies for a directory of files
@@ -275,9 +296,8 @@ expressionDependencies <- function(e) {
     }
 
     # base case: call to :: or :::
-    if (fname %in% c("::", ":::")) (
+    if (fname %in% c("::", ":::"))
       return(as.character(e[[2L]]))
-    )
 
     # base case: methods functions
     if (fname %in% c("setClass", "setRefClass", "setMethod", "setGeneric")) {
@@ -291,29 +311,66 @@ expressionDependencies <- function(e) {
   unique(unlist(children))
 }
 
-isRPackage <- function(project) {
-  file <- file.path(project, "DESCRIPTION")
-  if (!file.exists(file)) {
-    return(FALSE)
-  }
+# Read a DESCRIPTION file into a data.frame
+readDESCRIPTION <- function(path) {
 
-  DESCRIPTION <- tryCatch(
-    readDcf(file = file),
+  if (!file.exists(path))
+    stop("No DESCRIPTION file at path '", path, "'")
+
+  tryCatch(
+    readDcf(file = path, all = TRUE),
     error = function(e) {
-      warning("A 'DESCRIPTION' file was found, but could not be read as DCF",
-              call. = FALSE)
-      return(FALSE)
+      return(data.frame())
     }
   )
+}
+
+isRPackage <- function(project) {
+
+  descriptionPath <- file.path(project, "DESCRIPTION")
+  if (!file.exists(descriptionPath))
+    return(FALSE)
+
+  DESCRIPTION <- readDESCRIPTION(descriptionPath)
 
   # If 'Type' is missing from the DESCRIPTION file, then we implicitly assume
   # that it is an R package (#172)
-  if (!("Type" %in% colnames(DESCRIPTION))) {
+  if (!("Type" %in% names(DESCRIPTION)))
     return(TRUE)
-  }
 
   # Otherwise, ensure that the type is `Package`
-  Type <- unname(as.character(DESCRIPTION[, "Type"]))
+  Type <- unname(as.character(DESCRIPTION$Type))
   identical(Type, "Package")
 
 }
+
+# Infer whether a project is (implicitly) a Shiny application,
+# in the absence of explicit `library()` statements.
+isShinyApp <- function(project) {
+
+  # Check for a DESCRIPTION file with 'Type: Shiny'
+  descriptionPath <- file.path(project, "DESCRIPTION")
+  if (file.exists(descriptionPath)) {
+    DESCRIPTION <- readDESCRIPTION(descriptionPath)
+    if (length(DESCRIPTION$Type) && tolower(DESCRIPTION$Type) == "shiny")
+      return(TRUE)
+  }
+
+  # Check for a server.r with a 'shinyServer' call
+  serverPath <- file.path(project, "server.R")
+  if (file.exists(file.path(project, "server.R"))) {
+    contents <- paste(readLines(serverPath), collapse = "\n")
+    if (grepl("shinyServer\\s*\\(", contents, perl = TRUE))
+      return(TRUE)
+  }
+
+  # Check for a single-file application with 'app.R'
+  appPath <- file.path(project, "app.R")
+  if (file.exists(appPath)) {
+    contents <- paste(readLines(appPath), collapse = "\n")
+    if (grepl("shinyApp\\s*\\(", contents, perl = TRUE))
+      return(TRUE)
+  }
+
+  return(FALSE)
+}
